@@ -1,30 +1,12 @@
 #![cfg(feature = "full")]
 
 use {
-    crate::{decode_error::DecodeError, instruction::Instruction},
+    crate::{feature_set::FeatureSet, instruction::Instruction, precompiles::PrecompileError},
     bytemuck::{bytes_of, Pod, Zeroable},
     ed25519_dalek::{ed25519::signature::Signature, Signer, Verifier},
-    thiserror::Error,
+    std::sync::Arc,
 };
 
-#[derive(Error, Debug, Clone, PartialEq)]
-pub enum Ed25519Error {
-    #[error("ed25519 public key is not valid")]
-    InvalidPublicKey,
-    #[error("ed25519 signature is not valid")]
-    InvalidSignature,
-    #[error("offset not valid")]
-    InvalidDataOffsets,
-    #[error("instruction is incorrect size")]
-    InvalidInstructionDataSize,
-}
-
-impl<T> DecodeError<T> for Ed25519Error {
-    fn type_of() -> &'static str {
-        "Ed25519Error"
-    }
-}
-
 pub const PUBKEY_SERIALIZED_SIZE: usize = 32;
 pub const SIGNATURE_SERIALIZED_SIZE: usize = 64;
 pub const SIGNATURE_OFFSETS_SERIALIZED_SIZE: usize = 14;
@@ -97,20 +79,23 @@ pub fn new_ed25519_instruction(keypair: &ed25519_dalek::Keypair, message: &[u8])
     }
 }
 
-pub fn verify_signatures(data: &[u8], instruction_datas: &[&[u8]]) -> Result<(), Ed25519Error> {
+pub fn verify(
+    data: &[u8],
+    instruction_datas: &[&[u8]],
+    _feature_set: &Arc<FeatureSet>,
+) -> Result<(), PrecompileError> {
     if data.len() < SIGNATURE_OFFSETS_START {
-        return Err(Ed25519Error::InvalidInstructionDataSize);
+        return Err(PrecompileError::InvalidInstructionDataSize);
     }
     let num_signatures = data[0] as usize;
     if num_signatures == 0 && data.len() > SIGNATURE_OFFSETS_START {
-        return Err(Ed25519Error::InvalidInstructionDataSize);
+        return Err(PrecompileError::InvalidInstructionDataSize);
     }
     let expected_data_size = num_signatures
         .saturating_mul(SIGNATURE_OFFSETS_SERIALIZED_SIZE)
         .saturating_add(SIGNATURE_OFFSETS_START);
-    // We do not check or use the byte at data[1]
     if data.len() < expected_data_size {
-        return Err(Ed25519Error::InvalidInstructionDataSize);
+        return Err(PrecompileError::InvalidInstructionDataSize);
     }
     for i in 0..num_signatures {
         let start = i
@@ -120,7 +105,7 @@ pub fn verify_signatures(data: &[u8], instruction_datas: &[&[u8]]) -> Result<(),
 
         // bytemuck wants structures aligned
         let offsets: &Ed25519SignatureOffsets = bytemuck::try_from_bytes(&data[start..end])
-            .map_err(|_| Ed25519Error::InvalidDataOffsets)?;
+            .map_err(|_| PrecompileError::InvalidDataOffsets)?;
 
         // Parse out signature
         let signature = get_data_slice(
@@ -132,7 +117,7 @@ pub fn verify_signatures(data: &[u8], instruction_datas: &[&[u8]]) -> Result<(),
         )?;
 
         let signature = ed25519_dalek::Signature::from_bytes(signature)
-            .map_err(|_| Ed25519Error::InvalidSignature)?;
+            .map_err(|_| PrecompileError::InvalidSignature)?;
 
         // Parse out pubkey
         let pubkey = get_data_slice(
@@ -144,7 +129,7 @@ pub fn verify_signatures(data: &[u8], instruction_datas: &[&[u8]]) -> Result<(),
         )?;
 
         let publickey = ed25519_dalek::PublicKey::from_bytes(pubkey)
-            .map_err(|_| Ed25519Error::InvalidPublicKey)?;
+            .map_err(|_| PrecompileError::InvalidPublicKey)?;
 
         // Parse out message
         let message = get_data_slice(
@@ -157,7 +142,7 @@ pub fn verify_signatures(data: &[u8], instruction_datas: &[&[u8]]) -> Result<(),
 
         publickey
             .verify(message, &signature)
-            .map_err(|_| Ed25519Error::InvalidSignature)?;
+            .map_err(|_| PrecompileError::InvalidSignature)?;
     }
     Ok(())
 }
@@ -168,13 +153,13 @@ fn get_data_slice<'a>(
     instruction_index: u16,
     offset_start: u16,
     size: usize,
-) -> Result<&'a [u8], Ed25519Error> {
+) -> Result<&'a [u8], PrecompileError> {
     let instruction = if instruction_index == u16::MAX {
         data
     } else {
         let signature_index = instruction_index as usize;
         if signature_index >= instruction_datas.len() {
-            return Err(Ed25519Error::InvalidDataOffsets);
+            return Err(PrecompileError::InvalidDataOffsets);
         }
         &instruction_datas[signature_index]
     };
@@ -182,7 +167,7 @@ fn get_data_slice<'a>(
     let start = offset_start as usize;
     let end = start.saturating_add(size);
     if end > instruction.len() {
-        return Err(Ed25519Error::InvalidDataOffsets);
+        return Err(PrecompileError::InvalidDataOffsets);
     }
 
     Ok(&instruction[start..end])
@@ -190,12 +175,23 @@ fn get_data_slice<'a>(
 
 #[cfg(test)]
 pub mod test {
-    use super::*;
+    use {
+        super::*,
+        crate::{
+            ed25519_instruction::new_ed25519_instruction,
+            feature_set::FeatureSet,
+            hash::Hash,
+            signature::{Keypair, Signer},
+            transaction::Transaction,
+        },
+        rand::{thread_rng, Rng},
+        std::sync::Arc,
+    };
 
     fn test_case(
         num_signatures: u16,
         offsets: &Ed25519SignatureOffsets,
-    ) -> Result<(), Ed25519Error> {
+    ) -> Result<(), PrecompileError> {
         assert_eq!(
             bytemuck::bytes_of(offsets).len(),
             SIGNATURE_OFFSETS_SERIALIZED_SIZE
@@ -205,7 +201,11 @@ pub mod test {
         instruction_data[0..SIGNATURE_OFFSETS_START].copy_from_slice(bytes_of(&num_signatures));
         instruction_data[SIGNATURE_OFFSETS_START..DATA_START].copy_from_slice(bytes_of(offsets));
 
-        verify_signatures(&instruction_data, &[&[0u8; 100]])
+        verify(
+            &instruction_data,
+            &[&[0u8; 100]],
+            &Arc::new(FeatureSet::all_enabled()),
+        )
     }
 
     #[test]
@@ -219,8 +219,12 @@ pub mod test {
         instruction_data.truncate(instruction_data.len() - 1);
 
         assert_eq!(
-            verify_signatures(&instruction_data, &[&[0u8; 100]]),
-            Err(Ed25519Error::InvalidInstructionDataSize)
+            verify(
+                &instruction_data,
+                &[&[0u8; 100]],
+                &Arc::new(FeatureSet::all_enabled()),
+            ),
+            Err(PrecompileError::InvalidInstructionDataSize)
         );
 
         let offsets = Ed25519SignatureOffsets {
@@ -229,7 +233,7 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
 
         let offsets = Ed25519SignatureOffsets {
@@ -238,7 +242,7 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
 
         let offsets = Ed25519SignatureOffsets {
@@ -247,7 +251,7 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
     }
 
@@ -258,7 +262,10 @@ pub mod test {
             message_data_size: 1,
             ..Ed25519SignatureOffsets::default()
         };
-        assert_eq!(test_case(1, &offsets), Err(Ed25519Error::InvalidSignature));
+        assert_eq!(
+            test_case(1, &offsets),
+            Err(PrecompileError::InvalidSignature)
+        );
 
         let offsets = Ed25519SignatureOffsets {
             message_data_offset: 100,
@@ -267,7 +274,7 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
 
         let offsets = Ed25519SignatureOffsets {
@@ -277,7 +284,7 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
 
         let offsets = Ed25519SignatureOffsets {
@@ -287,7 +294,7 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
     }
 
@@ -299,7 +306,7 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
 
         let offsets = Ed25519SignatureOffsets {
@@ -308,7 +315,7 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
     }
 
@@ -320,7 +327,7 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
 
         let offsets = Ed25519SignatureOffsets {
@@ -329,7 +336,37 @@ pub mod test {
         };
         assert_eq!(
             test_case(1, &offsets),
-            Err(Ed25519Error::InvalidDataOffsets)
+            Err(PrecompileError::InvalidDataOffsets)
         );
     }
+
+    #[test]
+    fn test_ed25519() {
+        solana_logger::setup();
+
+        let privkey = ed25519_dalek::Keypair::generate(&mut thread_rng());
+        let message_arr = b"hello";
+        let mut instruction = new_ed25519_instruction(&privkey, message_arr);
+        let mint_keypair = Keypair::new();
+        let feature_set = Arc::new(FeatureSet::all_enabled());
+
+        let tx = Transaction::new_signed_with_payer(
+            &[instruction.clone()],
+            Some(&mint_keypair.pubkey()),
+            &[&mint_keypair],
+            Hash::default(),
+        );
+
+        assert!(tx.verify_precompiles(&feature_set).is_ok());
+
+        let index = thread_rng().gen_range(0, instruction.data.len());
+        instruction.data[index] = instruction.data[index].wrapping_add(12);
+        let tx = Transaction::new_signed_with_payer(
+            &[instruction],
+            Some(&mint_keypair.pubkey()),
+            &[&mint_keypair],
+            Hash::default(),
+        );
+        assert!(tx.verify_precompiles(&feature_set).is_err());
+    }
 }
